from argparse import ArgumentParser, FileType
import numpy as np
import pandas as p
import statsmodels.formula.api as smf
import datetime as dt

epoch = dt.datetime(2017, 1, 1)

def harmonic(length, period, start = 0, harmonics = 1, prefix = ""):
    x = np.arange(length) + start
    y = np.ndarray((harmonics * 2, length), dtype = np.float64)
    ns = []
    nc = []
    for i in range(harmonics):
        ysi = np.sin(x/period * 2 * np.pi * (i + 1))
        yci = np.cos(x/period * 2 * np.pi * (i + 1))
        y[i] = ysi
        ns.append("{}sin{}".format(prefix, i + 1))
        y[i + harmonics] = yci
        nc.append("{}cos{}".format(prefix, i + 1))
    y = p.DataFrame.from_records(y.transpose(), columns = ns + nc)
    return y


def main():
    parser = ArgumentParser(description='Harmonic models of all clusters in specified pickle')
    parser.add_argument("-i", "--input",  dest="input",  
                        help = "input aggregated ICP pickle path",  
                        metavar="PATH", required = True,
                        type = FileType('rb'))
    parser.add_argument("-w", "--weather",  dest="weather",  
                        help = "input weather pickle path",  
                        metavar="PATH", required = True,
                        type = FileType('rb'))
    parser.add_argument("--weather-harmonics", dest="weather_harmonics",
                        help = "number of harmonics for weather; default: 2",
                        type = int, default = 2)
    args = parser.parse_args()
    
    # print(args)

    # print(harmonic(17, 8, 0, 2, "test"))
    # print(epoch)

    # print(p.read_pickle(args.input))
    wdat = p.read_pickle(args.weather).drop(['record_no', 'station'], axis = 1)

    wantedtimes = p.date_range(wdat.temp_timestamp.min(), wdat.temp_timestamp.max(), freq = "30 min")
    wantedtimes = p.Series(wantedtimes.round(freq = "30 min"))
    wanteddf = p.DataFrame(index = wantedtimes)
    wdat = wanteddf.join(wdat.set_index(['temp_timestamp']), how = 'left').drop('temp_date', axis = 1)
    wdat.index.name = "read_time"
    wdat = wdat.fillna(method = 'ffill', axis = 0)
    wdat['min_rolling'] = wdat['tmin_c'].rolling("1D").min() 
    wdat['max_rolling'] = wdat['tmax_c'].rolling("1D").max() 
    print(wdat)
    wharmstart = (wdat.index.min() - epoch).total_seconds() / (60 * 30)
    wharm = harmonic(wdat.shape[0], period = 365.25, start = wharmstart, 
                     harmonics = args.weather_harmonics)
    print(wharm)
    
    args.input.close()
    args.weather.close()


if __name__ == "__main__":
    main()
